'**************************************************
' vbScript config.vbs
' 	configuration script
'
' Portable CHDK Development Environment for Windows
'	hostlua & ptp
' Project Version 4.2
'
' Licence: GPL2
' (C)2016-2022 forum.chdk-treff.de
' Author : rudi
'**************************************************
'
'on first run:
'	set NPP settins
'	set german localization for Germany and Austria

'always:
'	create nppexec config files with chdkptp check
'	set NPP shortcut
'
'--------------------------------------------------
Option Explicit
'begin configuration
Dim title
title = "CHDK hostluaPortable configuration"
'get vba script directory
Dim objFileSystem, vbaDir, configDir
Set objFileSystem = CreateObject("Scripting.FileSystemObject")
vbaDir = objFileSystem.GetParentFolderName(WScript.ScriptFullName)
configDir = "\bin\config"

If Right(vbaDir, Len(configDir) ) <> configDir  Then
	MsgBox "Wrong script directory!" & vbCrLf & vbCrLf & "'" & vbaDir & "'", vbCritical, title & " - ERROR"
	Wscript.Quit
End If

Private Function replaceXmlAttributText(objFS, fileName, parseError)
	Const FILE_WRITE = 2
	Dim stxt, ntxt, content
	replaceXmlAttributText = False
	Select Case parseError.errorCode
	Case -1072896762	'xmldom error code
		stxt = "<"
		ntxt = "&lt;"
	Case Else
		Exit Function
	End Select
	If objFileSystem.FileExists(fileName) Then
		content = ""
		content = objFileSystem.OpenTextFile(fileName).ReadAll
		If (content <> "") And (Mid(content, parseError.filePos + 1, 1) = stxt) Then
			content = Left(content, parseError.filePos) & ntxt & Mid(content, parseError.filePos + Len(stxt) + 1)
			objFileSystem.OpenTextFile(fileName, FILE_WRITE, False).Write content
			replaceXmlAttributText = True
		End If
	End If
End Function

'preset used directories
Dim mainDir, nppDir, nppLinkName, nppLinkFile, nppFile, nppLocaleFullFile, _
	nppLocaleDefaultFile, nppContextMenuFile, nppShorcutsFile, nppConfigFile, _
	nppStylersFile, nppSModelFile, nppPluginsDir, nppAPIsDir, _
	nppAutoCompleteDir, nppAutoCompleteFile, nppFunctionListDir, nppFunctionListFile, _
	nppeConfigDir, nppeConfigTxt, nppeConfigIni, luaWordlist, libusbFile, _
	libusbFile2, hlVersionFile, hlIniFile, nluaDir
mainDir = Left(vbaDir, Len(vbaDir) - Len(configDir) + 1)
configDir = mainDir & "bin\config\"
luaWordlist = configDir & "LuaFUNC3.txt"
nppDir = mainDir & "npp\"
nppLinkName = "Start Notepad++"
nppLinkFile = mainDir & nppLinkName & ".lnk"
nppFile = nppDir & "notepad++.exe"
nppLocaleDefaultFile = nppDir & "nativeLang.xml"
nppContextMenuFile = nppDir & "contextMenu.xml"
nppShorcutsFile = nppDir & "shortcuts.xml"
nppConfigFile = nppDir & "config.xml"
nppStylersFile = nppDir & "stylers.xml"
nppSModelFile = nppDir & "stylers.model.xml"
nppPluginsDir = nppDir & "plugins\"
nppAPIsDir = nppPluginsDir & "APIs\"
nppAutoCompleteDir = nppDir & "autoCompletion\"
nppAutoCompleteFile = "lua.xml"
nppFunctionListDir = nppDir & "functionList\"
nppFunctionListFile = "lua.xml"
nppeConfigDir = nppPluginsDir & "Config\"
nppeConfigTxt = nppeConfigDir & "npes_saved.txt"
nppeConfigIni = nppeConfigDir & "NppExec.ini"
libusbFile = objFileSystem.GetSpecialFolder(1) & "\drivers\libusb0.sys"
libusbFile2 = objFileSystem.GetSpecialFolder(0) & "\SysNative\drivers\libusb0.sys"
hlVersionFile = mainDir & "version.txt"
hlIniFile = mainDir & "config.ini"
nluaDir = mainDir & "bin\nativelua\"

'set default config values
'sections: [?][0]=sectionName
Const sectionNppLocalization = 0, sectionChdkptp = 1
Dim sections(2)
sections(sectionNppLocalization) = "nppLocalization"
sections(sectionChdkptp) = "chdkptp"
'iniEntries [?][0]=sectionIdx, [?][1]=key [?][2]=value
Const iniWinLocaleId = 0, iniNppLocalizationFile = 1, iniExtChdkptpPath = 2
Const inieSectionIdx = 0, inieKey = 1, inieValue = 2
Dim iniEntries(3, 3)
iniEntries(iniWinLocaleId, inieSectionIdx) = sectionNppLocalization
iniEntries(iniWinLocaleId, inieKey) = "winLocaleID"
iniEntries(iniNppLocalizationFile, inieSectionIdx) = sectionNppLocalization
iniEntries(iniNppLocalizationFile, inieKey) = "nppLocalizationFile"
iniEntries(iniExtChdkptpPath, inieSectionIdx) = sectionChdkptp
iniEntries(iniExtChdkptpPath, inieKey) = "extChdkptpPath"

Const iniWinLocaleIdDefValue = 1031, iniNppLocalizationFileDefValue = "german.xml", _
	iniExtChdkptpPathDefValue = ""

Dim iniCount, i
iniCount = 0
'read config ini
If objFileSystem.FileExists(hlIniFile) Then
	'read config.ini
	Set fh = objFileSystem.OpenTextFile(hlIniFile)
	If Not fh is Nothing Then
		Dim line, pos, val, sidx
		sidx = -1
		Do While fh.AtEndOfStream <> True
			line = Trim(fh.ReadLine)
			pos = InStr(line, "=")
			If Left(line, 1) = "[" And Right(line, 1) = "]" Then
				'found section
				sidx = -1
				For i = 0 To UBound(sections) - 1
					If line = "[" & sections(i) & "]" Then
						sidx = i
						Exit For
					End If
				Next
			ElseIf InStr(line, ";") = 0 And sidx >= 0 And pos > 0 Then
				val = Trim(Mid(line, pos + 1))
				If Len(val) > 0 Then
					For i = 0 to UBound(iniEntries) - 1
						If iniEntries(i, inieSectionIdx) = sidx And iniEntries(i, inieKey) = Trim(Left(line, pos - 1)) Then
							If IsEmpty(iniEntries(i, inieValue)) Then iniCount = iniCount + 1
							Select Case iniEntries(i, inieKey)
							Case "winLocaleID"
								iniEntries(i, inieValue) = CInt(val)
							Case "nppLocalizationFile"
								iniEntries(i, inieValue) = CStr(Replace(val, """", ""))
							Case "extChdkptpPath"
								iniEntries(i, inieValue) = CStr(Replace(val, """", ""))
							End Select
							Exit For
						End If
					Next
				End If
			End If
		Loop
		fh.Close
		Set fh = nothing
	End If
End If
'set defaults if empty
If iniCount <> UBound(iniEntries) Then
	iniCount = 0
	For i = 0 to UBound(iniEntries) - 1
		If IsEmpty(iniEntries(i, inieValue)) Then
			Select Case iniEntries(i, inieKey)
			Case "winLocaleID"
				iniEntries(i, inieValue) = iniWinLocaleIdDefValue
				iniCount = iniCount + 1
			Case "nppLocalizationFile"
				iniEntries(i, inieValue) = iniNppLocalizationFileDefValue
				iniCount = iniCount + 1
			Case "extChdkptpPath"
				iniEntries(i, inieValue) = iniExtChdkptpPathDefValue
				iniCount = iniCount + 1
			End Select
		End If
	Next
Else
	iniCount = 0
End If
If iniCount > 0 Or Not objFileSystem.FileExists(hlIniFile) Then
	'write config.ini
	Dim fh, c_entry
	Set fh = objFileSystem.CreateTextFile(hlIniFile, True, False)
	For i = 0 To UBound(sections) - 1
		fh.WriteLine("[" & sections(i) & "]")
		For c_entry = 0 to UBound(iniEntries) - 1
			If iniEntries(c_entry, inieSectionIdx) = i Then
				Select Case iniEntries(c_entry, inieKey)
				Case "winLocaleID"
					fh.WriteLine(";default or wrong values set npp language to english")
					fh.WriteLine(";npp localization, for germany, german = 1031, german.xml")
					fh.WriteLine(";  decimal windows language ID")
					fh.WriteLine(";  ref: https://msdn.microsoft.com/en-us/goglobal/bb964664.aspx")
					fh.WriteLine("winLocaleID = " & iniEntries(c_entry, inieValue))
					fh.WriteLine("")
				Case "nppLocalizationFile"
					fh.WriteLine(";  npp localization filename, existing file from \npp\localization\")
					fh.WriteLine("nppLocalizationFile = """ & iniEntries(c_entry, inieValue) & """")
					fh.WriteLine("")
				Case "extChdkptpPath"
					fh.WriteLine(";external chdkptp path as string or """"")
					fh.WriteLine("extChdkptpPath = """ & iniEntries(c_entry, inieValue) & """")
				End Select
			ElseIf iniEntries(c_entry, inieSectionIdx) > i Then
				Exit For
			End If
		Next
	Next
	fh.Close
	Set fh = nothing
End If

nppLocaleFullFile = nppDir & "localization\" & iniEntries(iniNppLocalizationFile, inieValue)

If objFileSystem.FileExists(nppLinkFile) Then
	'remove old NPP shortcut
	objFileSystem.DeleteFile(nppLinkFile)
Else
	Dim objXmlDom, nNode, pNode, qNode, rNode, sNode, tNode
	'set NPP settings
	If objFileSystem.FileExists(nppConfigFile) Then
		Dim objAttributes, Attr
		Set objXmlDom = CreateObject("Microsoft.XMLDOM")
		'load npp config
		objXmlDom.load nppConfigFile
		'<GUIConfig name="NewDocDefaultSettings" format="0" encoding="0" lang="23" codepage="-1" openAnsiAsUTF8="no"/>
		Set nNode = objXmlDom.selectSingleNode ("/NotepadPlus/GUIConfigs/GUIConfig[@name='NewDocDefaultSettings']")
		If Not nNode is Nothing Then
			Set objAttributes = nNode.attributes
			For Each Attr in objAttributes
				If Attr.name = "format" Then Attr.value = "0"
				If Attr.name = "encoding" Then Attr.value = "0"
				If Attr.name = "lang" Then Attr.value = "23"
				If Attr.name = "codepage" Then Attr.value = "-1"
				If Attr.name = "openAnsiAsUTF8" Then Attr.value = "no"
			Next
			Set objAttributes = Nothing
			Set nNode = Nothing
		End If
		'<GUIConfig name="searchEngine" searchEngineChoice="1" searchEngineCustom=""/>
		Set nNode = objXmlDom.selectSingleNode ("/NotepadPlus/GUIConfigs/GUIConfig[@name='searchEngine']")
		If Not nNode is Nothing Then
			Set objAttributes = nNode.attributes
			For Each Attr in objAttributes
				If Attr.name = "searchEngineChoice" Then Attr.value = "1"
			Next
			Set objAttributes = Nothing
			Set nNode = Nothing
		End If
		'<GUIConfig name="noUpdate" intervalDays="15" nextUpdateDate="20080426">yes</GUIConfig>
		Set nNode = objXmlDom.selectSingleNode ("/NotepadPlus/GUIConfigs/GUIConfig[@name='noUpdate']")
		If Not nNode is Nothing Then
			If nNode.Text <> "yes" Then nNode.Text = "yes"
			Set nNode = Nothing
		End If
		'<GUIConfig name="SaveAllConfirm">no</GUIConfig>
		Set nNode = objXmlDom.selectSingleNode ("/NotepadPlus/GUIConfigs/GUIConfig[@name='SaveAllConfirm']")
		If Not nNode is Nothing Then
			If nNode.Text <> "no" Then nNode.Text = "no"
			Set nNode = Nothing
		Else
			Set nNode = objXmlDom.selectSingleNode ("/NotepadPlus/GUIConfigs")
			If Not nNode is Nothing Then
				Set pNode = objXmlDom.createElement("GUIConfig")
				If Not pNode is Nothing Then
					pNode.setAttribute "name", "SaveAllConfirm"
					pNode.Text = "no"
					nNode.appendChild pNode
					Set pNode = Nothing
				End If
				Set nNode = Nothing
			End If
		End If
		'save npp config
		objXmlDom.save nppConfigFile
		Set objXmlDom = nothing
	End If
	'remove UserDefinedCommands
	If objFileSystem.FileExists(nppShorcutsFile) Then
		Dim nNodes
		Set objXmlDom = CreateObject("Microsoft.XMLDOM")
		objXmlDom.load nppShorcutsFile
		Set nNodes = objXmlDom.selectNodes ("/NotepadPlus/UserDefinedCommands/Command")
		If Not nNodes is Nothing Then
			For Each nNode In nNodes
				nNode.parentNode.removeChild(nNode)
			Next
			Set nNodes = Nothing
		End If
		' add user commands
		Set nNode = objXmlDom.selectSingleNode ("/NotepadPlus/UserDefinedCommands")
		If Not nNode is Nothing Then
			Set pNode = objXmlDom.createElement("Command")
			If Not pNode is Nothing Then
				pNode.setAttribute "name", "Open CHDKPTP USAGE.TXT [en]"
				pNode.setAttribute "Ctrl", "no"
				pNode.setAttribute "Alt", "no"
				pNode.setAttribute "Shift", "no"
				pNode.setAttribute "Key", "0"
				pNode.Text = "$(NPP_DIRECTORY)\notepad++.exe $(NPP_DIRECTORY)\..\bin\chdkptp\USAGE.TXT"
				nNode.appendChild pNode
				Set pNode = Nothing
			End If
			Set pNode = objXmlDom.createElement("Command")
			If Not pNode is Nothing Then
				pNode.setAttribute "name", "Show Lua 5.1 Manual [en]"
				pNode.setAttribute "Ctrl", "no"
				pNode.setAttribute "Alt", "no"
				pNode.setAttribute "Shift", "no"
				pNode.setAttribute "Key", "0"
				pNode.Text = "$(NPP_DIRECTORY)\..\docs\lua\5.1-en\contents.html"
				nNode.appendChild pNode
				Set pNode = Nothing
			End If
			Set pNode = objXmlDom.createElement("Command")
			If Not pNode is Nothing Then
				pNode.setAttribute "name", "Show Lua 5.2 Manual [en]"
				pNode.setAttribute "Ctrl", "no"
				pNode.setAttribute "Alt", "no"
				pNode.setAttribute "Shift", "no"
				pNode.setAttribute "Key", "0"
				pNode.Text = "$(NPP_DIRECTORY)\..\docs\lua\5.2-en\contents.html"
				nNode.appendChild pNode
				Set pNode = Nothing
			End If
			Set pNode = objXmlDom.createElement("Command")
			If Not pNode is Nothing Then
				pNode.setAttribute "name", "Show Lua 5.3 Manual [en]"
				pNode.setAttribute "Ctrl", "no"
				pNode.setAttribute "Alt", "no"
				pNode.setAttribute "Shift", "no"
				pNode.setAttribute "Key", "0"
				pNode.Text = "$(NPP_DIRECTORY)\..\docs\lua\5.3-en\contents.html"
				nNode.appendChild pNode
				Set pNode = Nothing
			End If
			Set pNode = objXmlDom.createElement("Command")
			If Not pNode is Nothing Then
				pNode.setAttribute "name", "Zeige Lua 5.1 Dokumentation [de]"
				pNode.setAttribute "Ctrl", "no"
				pNode.setAttribute "Alt", "no"
				pNode.setAttribute "Shift", "no"
				pNode.setAttribute "Key", "0"
				pNode.Text = "$(NPP_DIRECTORY)\..\docs\lua\5.1-de\Lua-Dokumentation.htm"
				nNode.appendChild pNode
				Set pNode = Nothing
			End If
			Set nNode = Nothing
		End If
		objXmlDom.save nppShorcutsFile
		Set objXmlDom = nothing
	End If
	'add context menu items
	If objFileSystem.FileExists(nppContextMenuFile) Then
		Set objXmlDom = CreateObject("Microsoft.XMLDOM")
		objXmlDom.load nppContextMenuFile
		Set pNode = objXmlDom.selectSingleNode ("/NotepadPlus/ScintillaContextMenu")
		If Not pNode is Nothing Then
			Dim isNewMenu
			isNewMenu = true
			Set nNode = pNode.selectSingleNode ("//Item[@FolderName='CHDK hostlua']")
			Do While Not nNode is Nothing
				isNewMenu = false
				pNode.removeChild(nNode)
				Set nNode = pNode.selectSingleNode ("//Item[@FolderName='CHDK hostlua']")
			Loop
			Set nNodes = pNode.selectNodes ("//Item")
			If Not nNodes is Nothing Then
				For Each nNode In nNodes
					If (nNode.attributes.item(0).name = "id") And (nNode.attributes.item(0).value = "0") Then
						If Not IsNewMenu Then
							'remove old separator
							pNode.removeChild(nNode)
							IsNewMenu = true
						Else
							'clone separator
							Set tNode = nNode.cloneNode(false)
							pNode.insertBefore tNode, nNode
							Set tNode = objXmlDom.createElement("Item")
							tNode.setAttribute "FolderName", "CHDK hostlua"
							tNode.setAttribute "PluginEntryName", "NppExec"
							tNode.setAttribute "PluginCommandItemName", "CHDK Lua local"
							pNode.insertBefore tNode, nNode
							Set tNode = tNode.cloneNode(false)
							tNode.setAttribute "PluginCommandItemName", "CHDK Lua local keypad"
							pNode.insertBefore tNode, nNode
							Set tNode = tNode.cloneNode(false)
							tNode.setAttribute "PluginCommandItemName", "CHDK Lua remote"
							pNode.insertBefore tNode, nNode
							Set tNode = tNode.cloneNode(false)
							tNode.setAttribute "PluginCommandItemName", "CHDK Lua remote output"
							pNode.insertBefore tNode, nNode
							Set tNode = tNode.cloneNode(false)
							tNode.setAttribute "PluginCommandItemName", "CHDK PTP"
							pNode.insertBefore tNode, nNode
							Set tNode = tNode.cloneNode(false)
							tNode.setAttribute "PluginCommandItemName", "Lua local (chdkptp)"
							pNode.insertBefore tNode, nNode
							Set tNode = tNode.cloneNode(false)
							tNode.setAttribute "PluginCommandItemName", "Native Lua"
							pNode.insertBefore tNode, nNode
							Set tNode = Nothing
							Exit For
						End If
					End If
				Next
				objXmlDom.save nppContextMenuFile
			End If
			Set nNodes = Nothing
			Set pNode = Nothing
			Set nNode = Nothing
		End If
		Set objXmlDom = nothing
	End If
	'remove native localization
	If objFileSystem.FileExists(nppLocaleDefaultFile) Then objFileSystem.DeleteFile(nppLocaleDefaultFile)
	Dim localeID
	localeID = GetLocale
	If (localeID = iniEntries(iniWinLocaleId, inieValue)) And objFileSystem.FileExists(nppLocaleFullFile) Then
		objFileSystem.GetFile(nppLocaleFullFile).Copy(nppLocaleDefaultFile)
	End If
	'copy Lua emu autocomplete file
	If objFileSystem.FileExists(configDir & "ac_" & nppAutoCompleteFile) Then
		objFileSystem.CopyFile configDir & "ac_" & nppAutoCompleteFile, nppAutoCompleteDir & nppAutoCompleteFile
	End If
	'copy Lua functionList file
	If objFileSystem.FileExists(configDir & "fl_" & nppFunctionListFile) Then
		objFileSystem.CopyFile configDir & "fl_" & nppFunctionListFile, nppFunctionListDir & nppFunctionListFile
	End If
	'chdk keyword highlighting
	If Not objFileSystem.FileExists(nppStylersFile) Then objFileSystem.CopyFile nppSModelFile, nppStylersFile
	Dim wordlist
	wordlist = ""
	If objFileSystem.FileExists(luaWordlist) Then
		wordlist = objFileSystem.OpenTextFile(luaWordlist).ReadLine
	End If
	If wordlist <> "" Then
		'add to stylers.xml
		Set objXmlDom = CreateObject("Microsoft.XMLDOM")
		'!!! bugfix for wrong chars in npp stylers.xml, section "REBOL" has "<" and ">" in xml attribute, line: 965
		'!!! compensation by replacing "<" with "&lt;", ">" is autoreplaced by xmldom.load
		Do
			objXmlDom.load nppStylersFile
			If objXmlDom.parseError.errorCode = 0 Then Exit Do
		Loop Until Not replaceXmlAttributText(objFileSystem, nppStylersFile, objXmlDom.parseError)
		If objXmlDom.parseError.errorCode = 0 Then
			Set nNode = objXmlDom.selectSingleNode ("/NotepadPlus/LexerStyles/LexerType[@name='lua']/WordsStyle[@name='FUNC3']")
			If Not nNode is Nothing Then
				nNode.Text = wordlist
				objXmlDom.save nppStylersFile
			End If
		End If
		If (objXmlDom is Nothing) Or (nNode is Nothing) Then MsgBox "hostlua function list (LuaFUNC3.txt) not insert!", vbInformation, title
		Set nNode = Nothing
		Set objXmlDom = Nothing
	End If
End If

'--- allways ---
'check libusb driver
Dim libusb
libusb = objFileSystem.FileExists(libusbFile)
'check external chdkptp path
If Len(iniEntries(iniExtChdkptpPath, inieValue)) > 0 Then
	If Right(iniEntries(iniExtChdkptpPath, inieValue), 1) <> "\" Then iniEntries(iniExtChdkptpPath, inieValue) = iniEntries(iniExtChdkptpPath, inieValue) & "\"
	If Not objFileSystem.FileExists(iniEntries(iniExtChdkptpPath, inieValue) & "chdkptp.exe") Then iniEntries(iniExtChdkptpPath, inieValue) = ""
End If
'create nppexec config files
Dim iniFile
Set iniFile = objFileSystem.CreateTextFile(nppeConfigIni, True, False)
iniFile.WriteLine("[Console]")
iniFile.WriteLine("Visible=0")
iniFile.WriteLine("ShowHelp=0")
iniFile.WriteLine("Encoding=0")
iniFile.WriteLine("SaveOnExecute=1")
iniFile.WriteLine("CmdHistory=1")
iniFile.WriteLine("SaveCmdHistory=0")
iniFile.WriteLine("NoInternalMsgs=1")
iniFile.WriteLine("Font=ECF(6)0(18)90010(D)302013143006F007500720069006500720020004E006500770(56)")
iniFile.WriteLine("TextColorNormal=0(6)")
iniFile.WriteLine("TextColorError=A01010")
iniFile.WriteLine("TextColorMessage=208020")
iniFile.WriteLine("BackgroundColor=00")
iniFile.WriteLine("PrintMsgReady=1")
iniFile.WriteLine("CustomMsgReady= === READY =============================\r")
iniFile.WriteLine("[ConsoleOutputFilter]")
iniFile.WriteLine("RecognitionMask0=""?*""")
iniFile.WriteLine("RecognitionEffect0=""+E 80 80 80 +I -B -U""")
iniFile.WriteLine("Enable=1")
iniFile.WriteLine("[Options]")
iniFile.WriteLine("ScriptNppStart=""NppExecOnOpen""")
iniFile.WriteLine("ScriptNppExit=""NppExecOnClose""")
iniFile.WriteLine("ToolbarBtn=1")
iniFile.WriteLine("[Restore]")
iniFile.WriteLine("LastSelectedScript=1")
iniFile.WriteLine("[UserMenu]")
iniFile.WriteLine("NppMacrosSubmenu=1")
iniFile.WriteLine("0=""CHDK Lua local :: CHDK Lua local""")
iniFile.WriteLine("1=""CHDK Lua local keypad :: CHDK Lua local keypad""")
If libusb Then
	iniFile.WriteLine("2=""CHDK Lua remote :: CHDK Lua remote""")
	iniFile.WriteLine("3=""CHDK Lua remote output :: CHDK Lua remote output""")
	iniFile.WriteLine("4=""CHDK PTP :: CHDK PTP""")
	iniFile.WriteLine("5=""Lua local (chdkptp) :: Lua local (chdkptp)""")
End If
iniFile.WriteLine("6=""Native Lua :: Native Lua""")
iniFile.Close
Set iniFile = nothing

Dim version
version = ""
If objFileSystem.FileExists(hlVersionFile) Then
	version = objFileSystem.OpenTextFile(hlVersionFile).ReadLine
End If

Dim nluaversion
nluaversion = ""
If objFileSystem.FolderExists(nluaDir) Then
	Dim flist, fname
	Set flist = objFileSystem.GetFolder(nluaDir)
	For Each fname in flist.files
		fname = Mid(fname, Len(nluaDir) + 1)
		If (Left(fname, 4) = "luac") And (Right(fname, 4) = ".exe") Then
			nluaversion = Mid(fname, 4 + 1, Len(fname) - (4 + 4))
			Exit For
		End If
	Next
	Set flist = Nothing
End If

'create nppe txt
Dim txtFile
Set txtFile = objFileSystem.CreateTextFile(nppeConfigTxt, True, False)
txtFile.WriteLine("::CHDK Lua local")
txtFile.WriteLine("\ ""$(HL_NPESDIR)lua_local.npes""")
txtFile.WriteLine("::CHDK Lua local keypad")
txtFile.WriteLine("\ ""$(HL_NPESDIR)lua_local.npes"" keypad")
If libusb Then
	txtFile.WriteLine("::CHDK Lua remote")
	txtFile.WriteLine("\ ""$(HL_NPESDIR)lua_remote.npes""")
	txtFile.WriteLine("::CHDK Lua remote output")
	txtFile.WriteLine("\ ""$(HL_NPESDIR)lua_remote.npes"" output")
	txtFile.WriteLine("::CHDK PTP")
	txtFile.WriteLine("\ ""$(HL_NPESDIR)ptpcmd.npes""")
	txtFile.WriteLine("::Lua local (chdkptp)")
	txtFile.WriteLine("\ ""$(HL_NPESDIR)lua_chdkptp.npes""")
End If
txtFile.WriteLine("::Native Lua")
txtFile.WriteLine("\ ""$(HL_NPESDIR)lua_native.npes""")
txtFile.WriteLine("")
txtFile.WriteLine("::NppExecOnOpen")
txtFile.WriteLine("echo please wait ...")
txtFile.WriteLine("echo")
txtFile.WriteLine("set local MSG_LIBUSB = """"")
txtFile.WriteLine("set local MSG_CHDKPTP = """"")
txtFile.WriteLine("set local LIBUSBCONF = 0" & -libusb)
txtFile.WriteLine("set local MAINEND ~ strrfind ""$(NPP_DIRECTORY)"" ""\""")
txtFile.WriteLine("set local MAINDIR ~ substr 0 $(MAINEND) $(NPP_DIRECTORY)")
txtFile.WriteLine("set HL_MAINDIR = $(MAINDIR)\")
txtFile.WriteLine("set HL_BINDIR = $(HL_MAINDIR)bin\")
txtFile.WriteLine("set HL_NPESDIR = $(HL_BINDIR)npes\")
txtFile.WriteLine("set HL_CHDKPTPDIR = $(HL_BINDIR)chdkptp\")
txtFile.WriteLine("set HL_NLUADIR = $(HL_BINDIR)nativelua\")
txtFile.WriteLine("set HL_NLUAVER = " & nluaversion)
txtFile.WriteLine("ENV_SET PATH = $(HL_BINDIR)")
txtFile.WriteLine("set CON_STD_COL = 000000")
txtFile.WriteLine("set CON_ERR_COL = D00000")
txtFile.WriteLine("set CON_MSG_COL = 0000D0")
txtFile.WriteLine("NPE_CONSOLE v+ f+ a- o0 i0 --")
txtFile.WriteLine("NPE_CMDALIAS hl_sub = npp_exec ""$(HL_NPESDIR)sub.npes""")
txtFile.WriteLine("NPP_CONSOLE -")
txtFile.WriteLine("cmd /c getfilesize.cmd """ & libusbFile & """")
txtFile.WriteLine("NPP_CONSOLE +")
txtFile.WriteLine("if $(OUTPUTL) > 0 goto libusb_exist")
txtFile.WriteLine("NPP_CONSOLE -")
txtFile.WriteLine("cmd /c getfilesize.cmd """ & libusbFile2 & """")
txtFile.WriteLine("NPP_CONSOLE +")
txtFile.WriteLine("if $(OUTPUTL) <= 0 goto libusb_notexist")
txtFile.WriteLine(":libusb_exist")
txtFile.WriteLine("if $(LIBUSBCONF) != 0 goto libusb_end")
txtFile.WriteLine("set local MSG_LIBUSB = echo libusb driver found! & echo run 'config.cmd' again. & echo.")
txtFile.WriteLine("goto libusb_end")
txtFile.WriteLine(":libusb_notexist")
txtFile.WriteLine("if $(LIBUSBCONF) == 0 goto libusb_end")
txtFile.WriteLine("set local MSG_LIBUSB = echo libusb driver not found! & echo run 'config.cmd' again. & echo.")
txtFile.WriteLine(":libusb_end")
If Len(iniEntries(iniExtChdkptpPath, inieValue)) > 0 Then
	txtFile.WriteLine("NPP_CONSOLE -")
	txtFile.WriteLine("cmd /c getfilesize.cmd """ & iniEntries(iniExtChdkptpPath, inieValue) & "chdkptp.exe""")
	txtFile.WriteLine("NPP_CONSOLE +")
	txtFile.WriteLine("if $(OUTPUTL) <= 0 goto chdkptp_end")
	txtFile.WriteLine("set HL_CHDKPTPDIR = " & iniEntries(iniExtChdkptpPath, inieValue))
	txtFile.WriteLine("set local MSG_CHDKPTP = echo using external chdkptp on & echo '$(HL_CHDKPTPDIR)' & echo.")
	txtFile.WriteLine(":chdkptp_end")
End If
txtFile.WriteLine("ENV_UNSET PATH")
txtFile.WriteLine("CLS")
txtFile.WriteLine("if $(MSG_LIBUSB) == """" goto noMsgLibusb")
txtFile.WriteLine("con_colour fg = $(CON_ERR_COL)")
txtFile.WriteLine("cmd /c $(MSG_LIBUSB)")
txtFile.WriteLine(":noMsgLibusb")
txtFile.WriteLine("if $(MSG_CHDKPTP) == """" goto noMsgChdkptp")
txtFile.WriteLine("con_colour fg = $(CON_MSG_COL)")
txtFile.WriteLine("cmd /c $(MSG_CHDKPTP)")
txtFile.WriteLine(":noMsgChdkptp")
txtFile.WriteLine("con_colour fg = $(CON_STD_COL)")
txtFile.WriteLine("echo hostlua environment " & version & "initialized")
txtFile.WriteLine("echo")
txtFile.WriteLine("")
txtFile.WriteLine("::NppExecOnClose")
txtFile.WriteLine("unset CON_MSG_COL")
txtFile.WriteLine("unset CON_ERR_COL")
txtFile.WriteLine("unset CON_STD_COL")
txtFile.WriteLine("unset HL_NPESDIR")
txtFile.WriteLine("unset HL_BINDIR")
txtFile.WriteLine("unset HL_MAINDIR")
txtFile.Close
Set txtFile = nothing

'set NPP shortcut
Dim objShell, objShortcut
Set objShell = WScript.CreateObject("Wscript.Shell")
Set objShortcut = objShell.CreateShortcut(nppLinkFile)
objShortcut.TargetPath = nppFile
objShortcut.Description = "Notepad++ Portable"
objShortcut.WorkingDirectory = mainDir
objShortcut.IconLocation = nppFile & ", 0"
objShortcut.Save

Set objShortcut = nothing
Set objShell = nothing
Set objFileSystem = nothing

MsgBox "Done." & vbCrLf & vbCrLf & "Run CHDK hostluaPortable with link '" & nppLinkName & "'.", vbInformation, title
