'**************************************************
' vbScript stdeditor.vbs
'   set/remove hostluaPortable as windows standard editor
'
' Portable CHDK Development Environment for Windows
' hostlua & ptp
' Project Version 4.2
'
' Licence: GPL2
' (C)2018-2022 forum.chdk-treff.de
' Author : rudi
'**************************************************
Option Explicit

Const PROJECT_NAME = "'hostluaPortable'"
Const ACTION_NONE = 0
Const ACTION_SET = 1
Const ACTION_REMOVE = 2
Const ACTION_OHER_NPP = 3
Const NppRegKey = "HKLM\Software\Microsoft\Windows NT\CurrentVersion\Image File Execution Options\notepad.exe"
Const NppRegValue = "Debugger"

Dim title
title = PROJECT_NAME & ": Config Windows Standard Editor"

Private Function CurrDateTimeString
  Const TD_PATTERN = "^(\d+)\D(\d+)\D(\d+)\D(\d+)\D(\d+)\D(\d+)\s*(AM|PM)?$"
  Dim res, strDT, regEx, dtMatch
  res = ""
  strDT = Formatdatetime(Now, 0)
  Set regEx = New RegExp
  regEx.Global = True
  regEx.Pattern = TD_PATTERN
  If regEx.Test(strDT) Then
    Set dtMatch = regEx.execute(strDT)(0).SubMatches
    If dtMatch.Count = 7 Then
      res = "-" & dtMatch(2)  'year
      res = res & dtMatch(1) & dtMatch(0) 'month, day
      res = res & "-" & dtMatch(3) & dtMatch(4) & dtMatch(5) & dtMatch(6) 'hour, min, sec, AM/PM
    End If
    Set dtMatch = Nothing
  End If
  Set regEx = Nothing
  CurrDateTimeString = res
End Function

Private Function GetNppFileString(objFileSystem)
  'check also hostluaPortable environment
  Dim vbaDir, configDir, mainDir, nppDir, res
  vbaDir = objFileSystem.GetParentFolderName(WScript.ScriptFullName)
  configDir = "\bin\config"
  If Right(vbaDir, Len(configDir) ) <> configDir  Then
    MsgBox "Wrong script directory!" & vbCrLf & vbCrLf & "'" & vbaDir & "'", vbCritical, title & " - ERROR"
    Wscript.Quit
  End If
  mainDir = Left(vbaDir, Len(vbaDir) - Len(configDir) + 1)
  If not objFileSystem.FileExists(mainDir & "Start Notepad++.lnk") Then
    MsgBox "Run 'config.cmd' at first!", vbCritical, title & " - ERROR"
    Wscript.Quit
  End If
  nppDir = mainDir & "npp\"
  res = nppDir & "notepad++.exe"
  If not objFileSystem.FileExists(res) Then
    MsgBox PROJECT_NAME & " not found!" & vbCrLf & vbCrLf & "'" & nppFile & "'", vbCritical, title & " - ERROR"
    Wscript.Quit
  End If
  GetNppFileString = res
End Function

Private Function GetRegKeyValue(objShell)
  'hint: reading of not exist value raise an error
  On Error Resume Next
  GetRegKeyValue = objShell.RegRead(NppRegKey & "\" & NppRegValue)
  If Not((Err.Number = 0) Or (Err.Number = -2147024894)) Then
    MsgBox "Unknown Error! " & vbCRLF & Err.Number & " Srce: " & Err.Source & " Desc: " &  Err.Description, vbCritical, title & " - ERROR"
    WScript.Quit
  End If
  Err.Clear
  On Error goto 0
End Function

Private Function  GetCurrentRegState(objShell, NppFileString, ByRef NewAction)
  Dim res, readKeyValue
  NewAction = ACTION_SET
  readKeyValue = GetRegKeyValue(objShell)
  If len(readKeyValue) > 0 Then
    NewAction = ACTION_REMOVE
    If InStr(readKeyValue, NppFileString) = 0 Then
      NewAction = ACTION_OHER_NPP
    End If
  End If
  res = "Current State: " & PROJECT_NAME & " is "
  If NewAction = ACTION_SET Then
    res = res & "NOT "
  End If
  res = res & "Windos Standard Editor."
  If NewAction = ACTION_OHER_NPP Then
    res = res & vbCRLF & vbCRLF
    res = res & "ANOTHER Notepad++ is Windows Standard Editor!"
    res = res & vbCRLF & "Path: " & Left(readKeyValue, InStrRev(readKeyValue, """"))
  End If
  GetCurrentRegState = res
End Function

Private Function HasElevatedPermissions(objShell)
  'check elevated permissions with 'fsutil' (run only on elevated permissions)
  Dim cmd
  cmd = "fsutil dirty query " & objShell.ExpandEnvironmentStrings("%SystemDrive%")
  HasElevatedPermissions = (objShell.Run(cmd, 0, true) = 0)
End Function

'--------------------------------------
' main
'--------------------------------------
Dim Action
Action = ACTION_NONE
'saved action from restart with elevated permissions
If WScript.Arguments.Count = 1 Then
  Select Case CInt(WScript.Arguments(0))
  Case ACTION_SET
    Action = ACTION_SET
  Case ACTION_REMOVE
    Action = ACTION_REMOVE
  Case ACTION_OHER_NPP
    Action = ACTION_OHER_NPP
  End Select
End If

Dim objFS, objSH, NppFile, CurrStateText
Set objFS = CreateObject("Scripting.FileSystemObject")
Set objSH = WScript.CreateObject("WScript.Shell")
'get npp full path
NppFile = GetNppFileString(objFS)
If Action = ACTION_NONE Then
  Dim msgText, resMsg, exportFN
  'get current state and required additional information for message box
  CurrStateText = GetCurrentRegState(objSH, NppFile, Action)
  If Action = ACTION_OHER_NPP Then
    'backup registry filename
    exportFN = Left(WScript.ScriptName, InStr(WScript.ScriptName, ".vbs") - 1) & CurrDateTimeString & ".reg"
    CurrStateText = CurrStateText & vbCrLf & "(On YES: Backup old RegistryKey to '" & exportFN & "'!)"
  End If
  msgText = "SET"
  If Action = ACTION_REMOVE Then
    msgText = "REMOVE"
  End If
  resMsg = MsgBox(CurrStateText & vbCrLf & vbCrLf & msgText & " " & PROJECT_NAME & " as Windows Standard Editor now?", _
    vbYesNo + vbQuestion + vbDefaultButton2, title)
  If resMsg = vbNo Then
    'NO: abbort any changes
    Wscript.Quit
  End If
  'save reg with current permissions
  If Action = ACTION_OHER_NPP Then
    If objSH.Run("reg export """ & NppRegKey & """ """ & exportFN & """", 0, true) <> 0 Then
      MsgBox "ERROR on Export RegKey! " & vbCRLF & "Destination Path: '" & exportFN & "'", vbCritical, title & " - ERROR"
      Wscript.Quit
    End If
  End If

  If Not HasElevatedPermissions(objSH) Then
    'restart script to get elevated permissions and action as argument
    Set objSH = Nothing
    Set objSH = CreateObject("Shell.Application")
    objSH.ShellExecute "wscript.exe", "//nologo """ & WScript.ScriptFullName & """ " & Action, "", "runas", 0
    Set objSH = Nothing
    Wscript.Quit
  End If
End If
'remove registry key
If (Action = ACTION_REMOVE) Or (Action = ACTION_OHER_NPP) Then
  objSH.RegDelete NppRegKey & "\"
End If
'write registry key
If (Action = ACTION_SET) Or (Action = ACTION_OHER_NPP) Then
  objSH.RegWrite NppRegKey & "\" & NppRegValue, """" & nppFile & """ -notepadStyleCmdline -z", "REG_SZ"
End If
'show current state on done
Dim NewAction
CurrStateText = GetCurrentRegState(objSH, NppFile, NewAction)
If (Action = NewAction) Or ((Action = ACTION_OHER_NPP) And (NewAction = ACTION_SET)) Then
  MsgBox "Sorry, something goes wrong - no changes!" & vbCrLf & vbCrLf & CurrStateText, vbCritical, title & " - ERROR"
Else
  MsgBox "Done!" & vbCrLf & vbCrLf & CurrStateText, vbInformation, title
End If
'clean up
Set objSH = Nothing
Set objFS = Nothing
