'**************************************************
' vbScript update.vbs
' 	update script
'
' Portable CHDK Development Environment for Windows
'	hostlua & ptp
' Project Version 4.2
'
' Licence: GPL2
' (C)2016-2022 forum.chdk-treff.de
' Author : rudi
'**************************************************

Option Explicit

Const cMajor = 0, cMinor = 1, cRevision = 2

Private Function getFolderPath(strDir)
	getFolderPath = strDir
	If Right(strDir, 1) = "\" Then
		getFolderPath = Left(strDir, Len(strDir) - 1)
	End If
End Function

Private Function deleteFolder(objFS, strDir)
	deleteFolder = False
	Dim strFolder
	strFolder = getFolderPath(strDir)
	If objFS.FolderExists(strFolder) Then
		objFS.DeleteFolder strFolder, True
		deleteFolder = True
	End If
End Function

Private Function splitVersion(strVersion)
	Dim regEx, entry, list, idx, aRes(2)
	Set regEx = New RegExp
	regEx.Pattern = "\d+"
	regEx.Global = True
	Set list = regEx.Execute(strVersion)
	idx = cMajor
	For Each entry in list
		If idx <= cRevision Then aRes(idx) = cInt(entry.Value)
		idx = idx + 1
	Next
	Set list = Nothing
	Set regEx = Nothing
	If idx = cRevision + 1 Then splitVersion = aRes
End Function

Private Function compareVersions(aVersionOld, aVersioNew)
	compareVersions = False
	If aVersioNew(cMajor) > aVersionOld(cMajor) Then
		compareVersions = True
	ElseIf aVersioNew(cMajor) = aVersionOld(cMajor) Then
		If aVersioNew(cMinor) > aVersionOld(cMinor) Then
			compareVersions = True
		ElseIf (aVersioNew(cMinor) = aVersionOld(cMinor) And _
			aVersioNew(cRevision) > aVersionOld(cRevision)) Then
			compareVersions = True
		End If
	End If
End Function

Private Function checkZipControlFile(strDir, strName)
  Dim aValues(2), ctrlValue, ctrlFile
  checkZipControlFile = False
  ctrlFile = strDir & strName
  On Error Resume Next
  If objFileSystem.FileExists(ctrlFile) Then
    ctrlValue = objFileSystem.OpenTextFile(ctrlFile).ReadLine
    objFileSystem.DeleteFile(ctrlFile)
    If (ctrlValue > 0) And (getFolderValues(strDir, aValues) = 0) Then
      If Err.Number = 0 Then checkZipControlFile = (ctrlValue = Join(aValues, "7"))
    End If
  End If
End Function

Private Function getFolderValues(strDir, ByRef aValues)
  Dim listFolder, listSubFolder, listFiles, item
  On Error Resume Next
 	Set listFolder = objFileSystem.GetFolder(strDir)
	Set listFiles = listFolder.Files
 	For Each item In listFiles
    aValues(0) = aValues(0) + 1
    aValues(1) = aValues(1) + item.Size
	Next
	Set listSubFolder = listFolder.Subfolders
	For Each item In listSubFolder
    aValues(2) = aValues(2) + 1
    getFolderValues = getFolderValues(item, aValues)
	Next
 	Set listFolder = nothing
	Set listFiles = nothing
	Set listSubFolder = nothing
  getFolderValues = Err.Number
End Function

'begin configuration
Dim titleUpd
titleUpd = "CHDK hostluaPortable update"
'get vba script directory
Dim objFileSystem, vbaDir, configDir
Set objFileSystem = CreateObject("Scripting.FileSystemObject")
vbaDir = objFileSystem.GetParentFolderName(WScript.ScriptFullName)
configDir = "\bin\config"
'check update script location
If Right(vbaDir, Len(configDir) ) <> configDir  Then
	MsgBox "Wrong script directory!" & vbCrLf & vbCrLf & "'" & vbaDir & "'", vbCritical, titleUpd & " - ERROR"
	WScript.Quit
End If
'presets and definitions
Dim mainDir, unzipDir, unzipFolderPath, updateDir, VersionTxtFile, hlVersionFile, updateVersionFile, _
	configIniFile, hlIniFile, updateIniFile, zipCtrlFile
mainDir = Left(vbaDir, Len(vbaDir) - Len(configDir) + 1)
configDir = mainDir & "bin\config\"
unzipDir = mainDir & "_update\"
updateDir = unzipDir & "hostluaPortable\"
VersionTxtFile = "version.txt"
hlVersionFile = mainDir & VersionTxtFile
updateVersionFile = updateDir & VersionTxtFile
unzipFolderPath = getFolderPath(unzipDir)
configIniFile = "config.ini"
hlIniFile = mainDir & configIniFile
updateIniFile = updateDir & configIniFile
zipCtrlFile = "zipinfo.txt"
'delete unused zip control file in main directory
If objFileSystem.FileExists(mainDir & zipCtrlFile) Then objFileSystem.DeleteFile(mainDir & zipCtrlFile)
'get current version; 4.1.004 = major.minor.revision
Dim oldVersion, newVersion
If objFileSystem.FileExists(hlVersionFile) Then
	oldVersion = splitVersion(objFileSystem.OpenTextFile(hlVersionFile).ReadLine)
End If
If IsEmpty(oldVersion) Then
	MsgBox "Wrong hostlua version!" & vbCrLf & vbCrLf & "Exit update now.", vbCritical, titleUpd & " - ERROR"
	WScript.Quit
End If
'find update file and check update filename
Dim listFiles, objFile, latestVersion, updateFile, updateFileExist, nppLinkFile
updateFileExist = False
Set listFiles = objFileSystem.GetFolder(mainDir).Files
For Each objFile in listFiles
	If Right(objFile.Name, 4) = ".zip" Then newVersion = splitVersion(objFile.Name) Else newVersion = vbEmpty
	If IsArray(newVersion) Then
		updateFileExist = True
		If compareVersions(oldVersion, newVersion) Then
			If IsEmpty(latestVersion) Then
				latestVersion = newVersion
				updateFile = objFile.Path
			ElseIf compareVersions(latestVersion, newVersion) Then
				latestVersion = newVersion
				updateFile = objFile.Path
			End If
		End If
	ElseIf objFile.Name = "Start Notepad++.lnk" Then
		nppLinkFile = objFile.Path
	End If
Next
'is exist update file?
If IsEmpty(updateFile) Then
	Dim newerText
	If updateFileExist Then newerText = " newer" Else newerText = ""
	MsgBox "No" & newerText & " hostlua update file found!" & vbCrLf & vbCrLf & "Exit update now.", vbCritical, titleUpd
	WScript.Quit
End If
'create unzip directory
deleteFolder objFileSystem, unzipFolderPath
objFileSystem.CreateFolder(unzipFolderPath)
'unzip update file
Dim objShellApp, objSource, objTarget
Set objShellApp = CreateObject( "Shell.Application" )
Set objSource = objShellApp.NameSpace(updateFile).Items()
Set objTarget = objShellApp.NameSpace(unzipFolderPath)
On Error Resume Next
objTarget.CopyHere objSource, &h100
If (Err.Number <> 0) Or Not checkZipControlFile(updateDir, zipCtrlFile) Then
	deleteFolder objFileSystem, unzipFolderPath
	msgText = "Error #" & CStr(Err.Number) & ", " & Err.Description
	If Err.Number = 0 Then msgText = "Unzipping canceled."
	MsgBox "Fatal error on extract zip file!" & vbCrLf & msgText & vbCrLf & vbCrLf & "Exit update now.", vbCritical, titleUpd & " - ERROR"
	WScript.Quit
End If
Set objShellApp = Nothing
'get and check update version from zip
Dim updateVersion, msgText
msgText = "Wrong hostlua version for update!"
If objFileSystem.FileExists(updateVersionFile) Then
	updateVersion = splitVersion(objFileSystem.OpenTextFile(updateVersionFile).ReadLine)
	If Not IsEmpty(updateVersion) And latestVersion(cMajor) = updateVersion(cMajor) And _
		latestVersion(cMinor) = updateVersion(cMinor) And _
		latestVersion(cRevision) = updateVersion(cRevision) Then
		msgText = ""
	Else
		msgText = "Wrong hostlua zip file!"
	End If
End If
'any error?
If Len(msgText) > 0 Then
	MsgBox msgText & vbCrLf & vbCrLf & "Exit update now.", vbCritical, titleUpd & " - ERROR"
	deleteFolder objFileSystem, unzipFolderPath
	WScript.Quit
End If
'start update?
If MsgBox ("current version " & oldVersion(cMajor) & "." & oldVersion(cMinor) & "." & Right("00" & oldVersion(cRevision),3) & vbCRLF & _
		"Start update to version " & updateVersion(cMajor) & "." & updateVersion(cMinor) & "." & Right("00" & updateVersion(cRevision), 3) & " now?" & vbCrLf & vbCrLf & _
		"Make sure Notepad++ is closed!", vbQuestion + vbYesNo, titleUpd) <> vbYes Then
	deleteFolder objFileSystem, unzipFolderPath
	MsgBox "Update abort.", vbCritical, titleUpd
	WScript.Quit
End If
'don't override config.ini
If objFileSystem.FileExists(hlIniFile) And objFileSystem.FileExists(updateIniFile) Then
	objFileSystem.DeleteFile(updateIniFile)
End If
'copy files from updateDir
On Error Resume Next
objFileSystem.CopyFolder getFolderPath(updateDir), getFolderPath(mainDir), True
If Err.Number <> 0 Then
	deleteFolder objFileSystem, unzipFolderPath
	MsgBox "Fatal error on update files!" & vbCrLf & "Error #" & CStr(Err.Number) & ", " & Err.Description & vbCrLf & vbCrLf & "Exit update now.", vbCritical, titleUpd & " - ERROR"
	WScript.Quit
End If
'remove old NPP shortcut
If Not IsEmpty(nppLinkFile) And objFileSystem.FileExists(nppLinkFile) Then
	objFileSystem.DeleteFile(nppLinkFile)
End If
'clean up
deleteFolder objFileSystem, unzipFolderPath
'run config
Dim fh, configScript
Set fh = objFileSystem.OpenTextFile(configDir & "config.vbs")
configScript = fh.ReadAll
fh.Close
Set fh = Nothing
'delete 'config done' message
configScript = Mid(configScript, 1, InStrRev(configScript, "MsgBox") - 1)
Execute configScript

MsgBox "Update done." & vbCrLf & vbCrLf & "Run CHDK hostluaPortable with link '" & nppLinkName & "'.", vbInformation, titleUpd
