--[[
**************************************************
 REMOTE LIBRARY "lremote.lua"
 set script defaults and run script remote

 Portable CHDK Development Environment for Windows
	hostlua & ptp
 Project Version 4.2

 Licence: GPL2
 (C)2013-2022 forum.chdk-treff.de
 Author : rudi
**************************************************
usage:
    luar require("lremote").exec(FULL_SCRIPTNAME, OUTPUT_TO_PTP)
    OUTPUT_TO_PTP = true or false
]]

lremote = {}

lremote.version = 4

local autoredraw = 1
local cam_print, cam_set_console_autoredraw

local function init_chdk_header_vars(filename)
    local chdk_version = {0, 0, 0, 0}
    for line in io.lines(filename) do
        local var, val=line:match("%s*@default%s*([A-Za-z])%s*([0-9]+)")
        if var and val then rawset(_G, var, tonumber(val))
        else
            local s = line:match("%s*@chdk_version%s*([%d%.]+)")
            if s then
                local cv = {}
                for v in string.gmatch(s, "%d+") do cv[#cv + 1] = tonumber(v) end
                if s:sub(1, 1) ~= "." and s:sub(-1) ~= "." and #cv > 1 and #cv <= 4 then
                    chdk_version = cv
                end
            else
                local var, val, val_range, var_ext = line:match('^%s*#([%w_]+)=(%-?%d+)%s+"[^"]+"%s+{([%w%p%s]+)}%s*(%a*)')
                if var and val and val_range and var_ext then
                    if var_ext == "table" then
                        local tab_val = {}
                        for _val in string.gmatch(val_range, "%S+") do
                            if #_val > 0 then tab_val[#tab_val + 1] = _val end
                        end
                        tab_val.index = tonumber(val)
                        tab_val.value = tab_val[tab_val.index]
                        rawset(_G, var, tab_val)
                    else
                        rawset(_G, var, tonumber(val))
                    end
                else
                    local var, val = line:match('^%s*#([%w_]+)=([-%w]+)%s+"[^"]+"')
                    if var and val then
                        if tonumber(val) then rawset(_G, var, tonumber(val))
                        elseif val == "true" then rawset(_G, var, true)
                        elseif val == "false" then rawset(_G, var, false)
                        end
                    end
                end
            end
        end
    end
    return chdk_version
end

local function write_ptp_str(...)
		write_usb_msg(...)
		--write_usb_msg() need a little bit time for write, otherwise lost lines
		sleep(50)
end

local function ptp_print(...)
    if autoredraw >= 0 then
		local str = ""
		for i = 1, select("#", ...) do
			if #str > 0 then str = str .. " " end
			str = str .. tostring(select(i, ...))
		end
		write_ptp_str(str)
	end
    cam_print(...)
end

local function ptp_set_console_autoredraw(val)
	autoredraw = val
    cam_set_console_autoredraw(val)
end

function lremote.exec(filename, output)
    local chdk_version = init_chdk_header_vars(filename)
    if output then
		cam_print, print = print, ptp_print
		cam_set_console_autoredraw, set_console_autoredraw = set_console_autoredraw, ptp_set_console_autoredraw
	end
    if chdk_version[1] == 1 and chdk_version[2] == 3 then
        if output then write_ptp_str("load wrap13.lua") end
        require("wrap13")
    end
    local script, loaderr = loadfile(filename)
    if script then script() else print(loaderr) end
end

return lremote
