--[[
**************************************************
 print chdkptp Lua functions list

 Portable CHDK Development Environment for Windows
	hostlua & ptp
 Project Version 4.2

 Licence: GPL2
 (C)2017-2022 forum.chdk-treff.de
 Author : rudi
**************************************************

  run with "Lua local (chdkptp)"
]]

assert(type(cmd) == 'table', "run with 'Lua local (chdkptp)'")

local key, t_val, val = 1, 2, 3
local readTable

local function getTabstop(colref)
	return string.rep(" ", (colref and 4 or 2))
end

local function writeVals(list, libname)
	if #list > 0 then
		local space = getTabstop(libname)
		print(string.format("%s--- %s ---", space, "values"))
		for i = 1, #list do
			print(string.format("%s %s%s = %s (%s)", space ,libname or "", list[i][key], list[i][val], list[i][t_val]))
		end
	end
end

local function writeFuncs(list, libname)
	if #list > 0 then
		local space = getTabstop(libname)
		print(string.format("%s--- %s ---", space, "functions"))
		for i = 1, #list do
			print(string.format("%s %s%s()", space , libname or "", list[i][key]))
		end
	end
end

local function writeTabs(list, libname)
	if #list > 0 then
		local space = getTabstop(libname)
		local islibname = libname ~= nil
		if islibname then
			print(string.format("%s--- %s ---", space, "table"))
		end
		for i = 1, #list do
			if list[i][1] ~= "_G" then
				if not islibname then print() end
				print(string.format("%s %s%s{ }", space, libname or "", list[i][key]))
				if not islibname then
				  readTable(list[i][val], list[i][key]..".")
				end
			end
		end
	end
end

function readTable(tab, libname)
	local vals, funcs, tabs = 1, 2, 3
	local list = {{}, {}, {}}

	for k, v in pairs(tab) do
		local tv = type(v)
		if tv == "table" then
			list[tabs][#list[tabs] + 1] = {k, tv, v}
		elseif tv == "function" then
			list[funcs][#list[funcs] + 1] = {k, tv, v}
		else
			list[vals][#list[vals] + 1] = {k, tv, string.gsub(tostring(v), "%s*%(%x+%)$", "")}
		end
	end

	table.sort(list[vals], function (a, b) return (a[1] < b[1]) end)
	table.sort(list[funcs], function (a, b) return (a[1] < b[1]) end)
	table.sort(list[tabs], function (a, b) return (a[1] < b[1]) end)

	writeVals(list[vals], libname)
	writeFuncs(list[funcs], libname)
	writeTabs(list[tabs], libname)
end

-- main
print(" Lua (chdkptp)")
print(" values, functions and tables:")
print(string.rep("=", 39))
print()
print(" global{ }")

readTable(_G)
