--[[
**************************************************
 show 'CHDK hostlua keypad' for configuration

 Portable CHDK Development Environment for Windows
	hostlua & ptp
 Project Version 4.2

 Licence: GPL2
 (C)2017-2022 forum.chdk-treff.de
 Author : rudi
**************************************************
]]

--[==============================================[
	msgbox(caption, message, [[[align], frame], redraw])

	caption: type string
	message: type table of strings
	align  : "left", "center", "right"; optional; default "left"
	frame  : 0 .. 3; optional; default 1
	redraw : boolean; optioanl; default false; true for handle by msgbox
--]==============================================]
function msgbox(caption, message, align, frame, redraw)
	-- prepare optional arguments
	local align_left, align_center, align_right = 1, 2, 3
	local t_align = {"left", "center", "right"}
	for i = 1, #t_align do
		if tostring(align) == t_align[i] then break	end
		if i == #t_align then align = t_align[align_left] end
	end
	frame = frame or 1
	redraw = redraw or false
	-- check arguments
	assert(type(caption) == "string", "wrong arg #1 for 'msgbox'")
	assert(type(message) == "table", "wrong arg #2 for 'msgbox'")
	assert(type(frame) == "number" and frame >= 0 and frame <= 3, "wrong arg #4 for 'msgbox'")
	assert(type(redraw) == "boolean", "wrong arg #5 for 'msgbox'")
	-- calculate text selective console window dimension
	local width, height = #caption, #message + 2
	for i = 1, #message do
		assert(type(message[i]) == "string", string.format("wrong type of message line %d for 'msgbox'", i))
		width = (#message[i] > width) and #message[i] or width
	end
	-- calculate console dimensions, max dimension 46 x 14
	local width_total = width + 2 * frame
	local height_total = height + 2 * frame
	local left = (46 - width_total) / 2
	local bottom = (14 - height_total) / 2
	local right = left + width_total
	local top = bottom + height_total
	-- set format string
	local sfmt = "%" .. frame .."s"
	local tfmt = "%s"
	if align == t_align[align_right] then
		tfmt = "%" .. width .. "s"
	end
	-- prepare console
	cls()
	set_console_layout(left, bottom, right, top)
	if redraw then set_console_autoredraw(0) end
	-- output function for printing line
	line_print = function(line)
		if #line == 0 then
			print()
		else
			if align == t_align[align_center] then
				sfmt = "%" .. (width_total - #line) / 2 .. "s"
			end
			print(string.format(sfmt .. tfmt, "", line))
		end
	end
	-- output function for print horizontal frame line
	hframe_print = function(count)
		for i = 1, count do line_print("") end
	end
	-- print console output
	hframe_print(frame)
	line_print(caption)
	line_print(string.rep("-", width))
	for i = 1, #message do line_print(message[i]) end
	hframe_print(frame)
	-- reset console
	console_redraw()
	if redraw then set_console_autoredraw(1) end
end

function restore()
	cls()
	set_console_layout(0, 0, 25, 5)
	set_console_autoredraw(1)
end

function get_msgtext()
	return "keypad configuration",
	{
		"press right mouse button",
		"to configure keypad",
		"",
		"[MENU] = exit"
	}
end

function main()
	local caption, text = get_msgtext()
	msgbox(caption, text, "center", 1, true)
	repeat
		wait_click(0)
	until is_key("menu")
	restore()
end

-- check 'CHDK Lua local [keypad]'
if get_buildinfo ~= nil and get_buildinfo().platform == "PC" and type(_keypad_info) == "function" then
	main()
else
	print("run with 'CHDK Lua local keypad'")
end
